/* dxdiagn
 *
 * Copyright 2003 TransGaming Technologies
 *
 * David Hammerton (david@transgaming.com)
 */

#include "windef.h"
#include "winbase.h"
#include "winnls.h"
#include "winreg.h"
#include "objbase.h"
#include "wine/debug.h"
#include "winerror.h"
#include "oleauto.h"

#include "dxdiag.h"
#include "dxdiagn_private.h"

/* I'm not really sure how the IID_IDxDiagContainer interface should be treated..
 * So I'm treating it as the interface to an undocumented class..
 * SO I guess this isn't really kosher COM.
 */

DEFAULT_DEBUG_CHANNEL(dxdiagn);

ICOM_VTABLE(IDxDiagContainer) dxDiagContainerVT;

HRESULT WINAPI DxDiagContainer_QueryInterface(PDXDIAGCONTAINER iface,
                                              REFIID riid, LPVOID *obj)
{
    ICOM_THIS(IDxDiagContainerImpl, iface);
    TRACE("(%p)->(%s, %p\n", iface, debugstr_guid(riid), obj);
    if (IsEqualGUID(&IID_IUnknown, riid) ||
        IsEqualGUID(&IID_IDxDiagContainer, riid))
    {
        This->ref++;
        *obj = This;
        return S_OK;
    }
    else
    {
        FIXME("(%p)->(%s, %p): no interface\n", iface, debugstr_guid(riid), obj);
        return E_NOINTERFACE;
    }

}

ULONG WINAPI DxDiagContainer_AddRef(PDXDIAGCONTAINER iface)
{
    ICOM_THIS(IDxDiagContainerImpl, iface);
    TRACE("(%p)->()\n", iface);
    return ++This->ref;
}

ULONG WINAPI DxDiagContainer_Release(PDXDIAGCONTAINER iface)
{
    ICOM_THIS(IDxDiagContainerImpl, iface);
    TRACE("(%p)->()\n", iface);
    return --This->ref; /* don't free us, static resource */
}

HRESULT WINAPI DxDiagContainer_GetChildContainer(PDXDIAGCONTAINER iface,
                                                 LPCWSTR pwszContainer,
                                                 IDxDiagContainer **ppInstance)
{
    ICOM_THIS(IDxDiagContainerImpl, iface);
    Container_SubContainer *childContainer;
    int i;

    TRACE("(%p)->(%s, %p)\n", iface, debugstr_w(pwszContainer), ppInstance);

    i = 0;
    do
    {
        childContainer = &This->scContainers[i];
        i++;
    } while (childContainer->pwszContainerName &&
             lstrcmpW(childContainer->pwszContainerName, pwszContainer));

    if (!childContainer->pwszContainerName)
        return E_INVALIDARG;

    DxDiagContainer_AddRef((IDxDiagContainer *)childContainer->pContainer);

    *ppInstance = (IDxDiagContainer *)childContainer->pContainer;

    return S_OK;
}

/*
 * There are 4 VARIANT property types.
 * The property name contains a hungarian notation prefix.
 *
 * VARTYPE   MEMBER    PREFIX
 * 
 *  VT_UI4    ulVal      dw
 *  VT_I4     lVal       ??
 *  VT_BOOL   boolVal    ??
 *  VT_BSTR   bstrVal    sz
 */

HRESULT WINAPI DxDiagContainer_GetProp(PDXDIAGCONTAINER iface,
                                       LPCWSTR pwszPropname,
                                       VARIANT *pvarProp)
{
    ICOM_THIS(IDxDiagContainerImpl, iface);
    Container_Property *property;
    int i;

    TRACE("(%p)->(%s, %p)\n", iface, debugstr_w(pwszPropname), pvarProp);

    i = 0;
    do
    {
        property = &This->pProperties[i];
        i++;
    } while (property->pwszPropName &&
             lstrcmpW(property->pwszPropName, pwszPropname));

    if (!property->pwszPropName)
        return E_INVALIDARG;

    switch (property->varType)
    {
        case DXD_DWORD:
            V_VT(pvarProp) = VT_UI4;
            V_UI4(pvarProp) = property->dw;
            break;
        case DXD_INT:
            V_VT(pvarProp) = VT_I4;
            V_UI4(pvarProp) = property->i;
            break;
        case DXD_BOOL:
            V_VT(pvarProp) = VT_BOOL;
            V_BOOL(pvarProp) = property->b;
            break;
        case DXD_STR:
            V_VT(pvarProp) = VT_BSTR;
            V_BSTR(pvarProp) = SysAllocString(property->psz);
            break;
        case DXD_EMPTY:
            FIXME("empty property??");
            return E_INVALIDARG;
    }

    return S_OK;
}

ICOM_VTABLE(IDxDiagContainer) dxDiagContainerVT =
{
    ICOM_MSVTABLE_COMPAT_DummyRTTIVALUE
    DxDiagContainer_QueryInterface,
    DxDiagContainer_AddRef,
    DxDiagContainer_Release,
    (void*)0xdead0004, /* GetNumberOfChildContainers */
    (void*)0xdead0005, /* EnumChildContainerNames */
    DxDiagContainer_GetChildContainer,
    (void*)0xdead0007, /* GetNumberOfProps */
    (void*)0xdead0008, /* EnumProps */
    DxDiagContainer_GetProp
};

